/*
http://www.two-sdg.demon.co.uk/curbralan/code/dirent/dirent.html

    Implementation of POSIX directory browsing functions and types for Win32.

    Author:  Kevlin Henney (kevlin@acm.org, kevlin@curbralan.com)
    History: Created March 1997. Updated June 2003.
    Rights:  See end of file.

*/

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "defs/defs.h"	//numberof
#include "winsystem/wstring.h"
#include "winsystem/dirent.h"

using namespace std;

#ifdef __cplusplus
extern "C"
{
#endif

DIR *opendir(const char *_name)
{
    DIR *dir = 0;
    if(_name && _name[0])
    {
        size_t base_length = strlen(_name);
		wstring all;
		wstring name = str2wstr(_name);

		all = /* search pattern must end with suitable wildcard */
            wcschr(str2wstr("/\\").c_str(), name[base_length - 1])
			? str2wstr("*")
			: str2wstr("/*");

        if((dir = (DIR *) malloc(sizeof *dir)) != 0 &&
           (dir->name = (wchar_t *) malloc((base_length + all.length() + 1)*sizeof(wchar_t))) != 0)
        {
            wcscat(wcscpy(dir->name, name.c_str()), all.c_str());

            if((dir->handle = FindFirstFile(dir->name, &dir->info)) != INVALID_HANDLE_VALUE)
            {
                dir->result.d_name[0] = 0;
            }
            else /* rollback */
            {
                free(dir->name);
                free(dir);
                dir = 0;
            }
        }
        else /* rollback */
        {
            free(dir);
            dir   = 0;
            errno = ENOMEM;
        }
    }
    else
    {
        errno = EINVAL;
    }

    return dir;
}

int closedir(DIR *dir)
{
    int result = -1;

    if(dir)
    {
        if(dir->handle != INVALID_HANDLE_VALUE)
        {
            result = FindClose(dir->handle);
        }

        free(dir->name);
        free(dir);
    }

    if(result == -1) /* map all errors to EBADF */
    {
        errno = EBADF;
    }

    return result;
}

struct dirent *readdir(DIR *dir)
{
    struct dirent *result = 0;

    if(dir && dir->handle != INVALID_HANDLE_VALUE)
    {
        if(!dir->result.d_name || FindNextFile(dir->handle, &dir->info))
        {
            result         = &dir->result;
			string s = wstr2str(dir->info.cFileName);
			strncpy(result->d_name, s.c_str(), numberof(result->d_name)-1);
        }
    }
    else
    {
        errno = EBADF;
    }

    return result;
}

void rewinddir(DIR *dir)
{
    if(dir && dir->handle != INVALID_HANDLE_VALUE)
    {
        FindClose(dir->handle);
        dir->handle = FindFirstFile(dir->name, &dir->info);
        dir->result.d_name[0] = 0;
    }
    else
    {
        errno = EBADF;
    }
}

#ifdef __cplusplus
}
#endif

/*

    Copyright Kevlin Henney, 1997, 2003. All rights reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for any purpose is hereby granted without fee, provided
    that this copyright and permissions notice appear in all copies and
    derivatives.
    
    This software is supplied "as is" without express or implied warranty.

    But that said, if there are any problems please get in touch.

*/
