#include <assert.h>
#include <windows.h>

#include <iostream>

#include "system/exception.h"
#include "winsystem/timer.h"

using namespace std;

//Thanks to ID software / Quake 1 source for the code

void WinHiResTimer::init() {
	//query performance frequency
	LARGE_INTEGER PerformanceFreqLI;
	if (!QueryPerformanceFrequency (&PerformanceFreqLI)) {
		throw Error() << "WinHiResTimer::init unable to read Performance Counter Frequency";
	}

	unsigned int lowpart = (unsigned int)PerformanceFreqLI.LowPart;
	unsigned int highpart = (unsigned int)PerformanceFreqLI.HighPart;
	performanceCounterLowShift = 0;

	while (highpart || (lowpart > 2000000.0)) {
		performanceCounterLowShift++;
		lowpart >>= 1;
		lowpart |= (highpart & 1) << 31;
		highpart >>= 1;
	}

	performanceFreq = 1.0 / (double)lowpart;

	//init class vars
	currentTime = 0;

	//clear the lastTime
	lastTime = 0;
	//and call getTime() once to correctly fill the lastTime variable
	getTime();
}

double WinHiResTimer::getTime() {

	LARGE_INTEGER PerformanceCount;
	QueryPerformanceCounter (&PerformanceCount);

	unsigned int thisTime;

	thisTime = ((unsigned int)PerformanceCount.LowPart >> performanceCounterLowShift) |
			   ((unsigned int)PerformanceCount.HighPart << (32 - performanceCounterLowShift));

	if (thisTime <= lastTime) {	//keep from resetting current time on turnover
		lastTime = thisTime;
	} else {
		unsigned int deltaTime = thisTime - lastTime;
		lastTime = thisTime;

		double deltaFloatTime = (double)deltaTime * performanceFreq;

		currentTime += deltaFloatTime;
	}

	return currentTime;
}
